@@ -1,5 +1,6 @@
 /*
  * synergy -- mouse and keyboard sharing utility
+ * Copyright (C) 2012 Bolton Software Ltd.
  * Copyright (C) 2008 Volker Lanz (vl@fidra.de)
  * 
  * This package is free software; you can redistribute it and/or
@@ -89,6 +90,7 @@ class MainWindow : public QMainWindow, public Ui::MainWindowBase
 		QString appPath(const QString& name);
 		void start(bool firstRun);
 		void clearLog();
+		VersionChecker& versionChecker() { return m_VersionChecker; }
 
 	public slots:
 		void appendLogRaw(const QString& text);
@@ -112,7 +114,6 @@ class MainWindow : public QMainWindow, public Ui::MainWindowBase
 		void logOutput();
 		void logError();
 		void updateFound(const QString& version);
-		void refreshApplyButton();
 
 	protected:
 		QSettings& settings() { return m_Settings; }
@@ -140,6 +141,9 @@ class MainWindow : public QMainWindow, public Ui::MainWindowBase
 		void stopService();
 		void stopDesktop();
 		void setFormEnabled(bool enabled);
+		void changeEvent(QEvent* event);
+		void retranslateMenuBar();
+		void updatePremiumInfo();
 
 	private:
 		QSettings& m_Settings;
@@ -150,12 +154,16 @@ class MainWindow : public QMainWindow, public Ui::MainWindowBase
 		QTemporaryFile* m_pTempConfigFile;
 		QSystemTrayIcon* m_pTrayIcon;
 		QMenu* m_pTrayIconMenu;
-		bool m_alreadyHidden;
-		VersionChecker m_versionChecker;
-		SetupWizard* m_SetupWizard;
+		bool m_AlreadyHidden;
+		VersionChecker m_VersionChecker;
 		IpcClient m_IpcClient;
 		bool m_ElevateProcess;
 		bool m_SuppressElevateWarning;
+		QMenuBar* m_pMenuBar;
+		QMenu* m_pMenuFile;
+		QMenu* m_pMenuEdit;
+		QMenu* m_pMenuWindow;
+		QMenu* m_pMenuHelp;
 
 private slots:
 	void on_m_pButtonApply_clicked();
