@@ -1,5 +1,6 @@
 /*
  * synergy -- mouse and keyboard sharing utility
+ * Copyright (C) 2012 Bolton Software Ltd.
  * Copyright (C) 2008 Volker Lanz (vl@fidra.de)
  * 
  * This package is free software; you can redistribute it and/or
@@ -49,29 +50,25 @@ int main(int argc, char* argv[])
 	if (!waitForTray())
 		return -1;
 
+#ifndef Q_OS_WIN
 	QApplication::setQuitOnLastWindowClosed(false);
-
-	QTranslator qtTranslator;
-	qtTranslator.load("qt_" + QLocale::system().name(),
-		QLibraryInfo::location(QLibraryInfo::TranslationsPath));
-	app.installTranslator(&qtTranslator);
-
-	QTranslator synergyTranslator;
-	synergyTranslator.load("res/lang/" + QLocale::system().name());
-	app.installTranslator(&synergyTranslator);
+#endif
 
 	QSettings settings;
 	AppConfig appConfig(&settings);
+
+	app.switchTranslator(appConfig.language());
+
 	MainWindow mainWindow(settings, appConfig);
 	SetupWizard setupWizard(mainWindow, true);
 
-	if (appConfig.wizardHasRun())
+	if (appConfig.wizardShouldRun())
 	{
-		mainWindow.start(false);
+		setupWizard.show();
 	}
 	else
 	{
-		setupWizard.show();
+		mainWindow.start(false);
 	}
 
 	return app.exec();
