@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2004 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2004 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,10 +23,12 @@
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 
+class IEventQueue;
+
 //! Event queue buffer for Win32
 class CMSWindowsEventQueueBuffer : public IEventQueueBuffer {
 public:
-	CMSWindowsEventQueueBuffer();
+	CMSWindowsEventQueueBuffer(IEventQueue* events);
 	virtual ~CMSWindowsEventQueueBuffer();
 
 	// IEventQueueBuffer overrides
@@ -37,11 +40,14 @@ public:
 						newTimer(double duration, bool oneShot) const;
 	virtual void		deleteTimer(CEventQueueTimer*) const;
 
+	virtual	void		cacheCurrentEventQueueRef() {}
+
 private:
 	DWORD				m_thread;
 	UINT				m_userEvent;
 	MSG					m_event;
 	UINT				m_daemonQuit;
+	IEventQueue*		m_events;
 };
 
 #endif
