@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2011 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2011 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +21,8 @@
 
 #define WIN32_LEAN_AND_MEAN
 #include <Windows.h>
-#include "CSynergyHook.h"
+#include "synwinhk.h"
+#include "synwinxt.h"
 
 //! Loads Windows hook DLLs.
 class CMSWindowsHookLibraryLoader
@@ -30,6 +32,7 @@ public:
 	virtual ~CMSWindowsHookLibraryLoader();
 
 	HINSTANCE			openHookLibrary(const char* name);
+	HINSTANCE			openShellLibrary(const char* name);
 
 	// TODO: either make these private or expose properly
 	InitFunc			m_init;
@@ -38,8 +41,7 @@ public:
 	SetZoneFunc			m_setZone;
 	SetModeFunc			m_setMode;
 
-private:
-	HINSTANCE			m_hookLibrary;
+	GetDraggingFileDir	m_getDraggingFileDir;
 };
 
 #endif
\ No newline at end of file
