@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2002 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2002 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,8 +36,13 @@
 #include "TMethodJob.h"
 #include "CArch.h"
 #include "CArchMiscWindows.h"
+#include "CApp.h"
+#include "CArgsBase.h"
+#include "CClientApp.h"
+#include "CClient.h"
 #include <string.h>
 #include <pbt.h>
+#include <Shlobj.h>
 
 //
 // add backwards compatible multihead support (and suppress bogus warning).
@@ -83,8 +89,9 @@ CMSWindowsScreen*		CMSWindowsScreen::s_screen   = NULL;
 CMSWindowsScreen::CMSWindowsScreen(
 	bool isPrimary,
 	bool noHooks,
-	const CGameDeviceInfo& gameDeviceInfo,
-	bool stopOnDeskSwitch) :
+	bool stopOnDeskSwitch,
+	IEventQueue* events) :
+	CPlatformScreen(events),
 	m_isPrimary(isPrimary),
 	m_noHooks(noHooks),
 	m_is95Family(CArchMiscWindows::isWindows95Family()),
@@ -108,11 +115,11 @@ CMSWindowsScreen::CMSWindowsScreen(
 	m_ownClipboard(false),
 	m_desks(NULL),
 	m_hookLibrary(NULL),
+	m_shellLibrary(NULL),
 	m_keyState(NULL),
 	m_hasMouse(GetSystemMetrics(SM_MOUSEPRESENT) != 0),
 	m_showingMouse(false),
-	m_gameDeviceInfo(gameDeviceInfo),
-	m_gameDevice(NULL)
+	m_events(events)
 {
 	assert(s_windowInstance != NULL);
 	assert(s_screen   == NULL);
@@ -120,23 +127,30 @@ CMSWindowsScreen::CMSWindowsScreen(
 	s_screen = this;
 	try {
 		if (m_isPrimary && !m_noHooks) {
-			m_hookLibrary = openHookLibrary("synrgyhk");
+			m_hookLibrary = openHookLibrary("synwinhk");
 		}
+		m_shellLibrary = openShellLibrary("synwinxt");
+
 		m_screensaver = new CMSWindowsScreenSaver();
 		m_desks       = new CMSWindowsDesks(
 							m_isPrimary, m_noHooks,
 							m_hookLibrary, m_screensaver,
-							*EVENTQUEUE,
+							m_events,
 							new TMethodJob<CMSWindowsScreen>(this,
 								&CMSWindowsScreen::updateKeysCB),
 							stopOnDeskSwitch);
-		m_keyState    = new CMSWindowsKeyState(m_desks, getEventTarget());
+		m_keyState    = new CMSWindowsKeyState(m_desks, getEventTarget(), m_events);
 		updateScreenShape();
 		m_class       = createWindowClass();
 		m_window      = createWindow(m_class, "Synergy");
 		forceShowCursor();
 		LOG((CLOG_DEBUG "screen shape: %d,%d %dx%d %s", m_x, m_y, m_w, m_h, m_multimon ? "(multi-monitor)" : ""));
 		LOG((CLOG_DEBUG "window is 0x%08x", m_window));
+
+		char desktopPath[MAX_PATH];
+		SHGetFolderPath(NULL, CSIDL_DESKTOP, NULL, 0, desktopPath);
+		m_desktopPath = CString(desktopPath);
+		LOG((CLOG_DEBUG "temporarily use desktop directory for drop target: %s", m_desktopPath.c_str()));
 	}
 	catch (...) {
 		delete m_keyState;
@@ -148,36 +162,20 @@ CMSWindowsScreen::CMSWindowsScreen(
 		if (m_hookLibrary != NULL)
 			closeHookLibrary(m_hookLibrary);
 
+		if (m_shellLibrary != NULL)
+			closeShellLibrary(m_shellLibrary);
+
 		s_screen = NULL;
 		throw;
 	}
 
 	// install event handlers
-	EVENTQUEUE->adoptHandler(CEvent::kSystem, IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(CEvent::kSystem, m_events->getSystemTarget(),
 							new TMethodEventJob<CMSWindowsScreen>(this,
 								&CMSWindowsScreen::handleSystemEvent));
 
 	// install the platform event queue
-	EVENTQUEUE->adoptBuffer(new CMSWindowsEventQueueBuffer);
-
-	if ((gameDeviceInfo.m_mode == CGameDeviceInfo::kGameModeXInput) &&
-		(gameDeviceInfo.m_poll != CGameDeviceInfo::kGamePollDynamic))
-		LOG((CLOG_WARN "only dynamic polling is supported with xnput."));
-
-	if ((gameDeviceInfo.m_mode == CGameDeviceInfo::kGameModeJoyInfoEx) &&
-		(gameDeviceInfo.m_poll != CGameDeviceInfo::kGamePollStatic))
-		LOG((CLOG_WARN "only static polling is supported with joyinfoex."));
-
-	if (m_gameDeviceInfo.m_mode == CGameDeviceInfo::kGameModeXInput) {
-#if GAME_DEVICE_SUPPORT
-		m_gameDevice = new CMSWindowsXInput(this, gameDeviceInfo);
-#else if _AMD64_
-		LOG((CLOG_WARN "xinput game device mode not supported for 64-bit."));
-#endif
-	}
-	else {
-		m_gameDevice = new CEventGameDevice(getEventTarget());
-	}
+	m_events->adoptBuffer(new CMSWindowsEventQueueBuffer(m_events));
 }
 
 CMSWindowsScreen::~CMSWindowsScreen()
@@ -185,20 +183,20 @@ CMSWindowsScreen::~CMSWindowsScreen()
 	assert(s_screen != NULL);
 
 	disable();
-	EVENTQUEUE->adoptBuffer(NULL);
-	EVENTQUEUE->removeHandler(CEvent::kSystem, IEventQueue::getSystemTarget());
+	m_events->adoptBuffer(NULL);
+	m_events->removeHandler(CEvent::kSystem, m_events->getSystemTarget());
 	delete m_keyState;
 	delete m_desks;
 	delete m_screensaver;
 	destroyWindow(m_window);
 	destroyClass(m_class);
 
-	if (m_gameDevice != NULL)
-		delete m_gameDevice;
-
 	if (m_hookLibrary != NULL)
 		closeHookLibrary(m_hookLibrary);
 
+	if (m_shellLibrary != NULL)
+		closeShellLibrary(m_shellLibrary);
+
 	s_screen = NULL;
 }
 
@@ -223,8 +221,8 @@ CMSWindowsScreen::enable()
 	assert(m_isOnScreen == m_isPrimary);
 
 	// we need to poll some things to fix them
-	m_fixTimer = EVENTQUEUE->newTimer(1.0, NULL);
-	EVENTQUEUE->adoptHandler(CEvent::kTimer, m_fixTimer,
+	m_fixTimer = m_events->newTimer(1.0, NULL);
+	m_events->adoptHandler(CEvent::kTimer, m_fixTimer,
 							new TMethodEventJob<CMSWindowsScreen>(this,
 								&CMSWindowsScreen::handleFixes));
 
@@ -281,8 +279,8 @@ CMSWindowsScreen::disable()
 
 	// uninstall fix timer
 	if (m_fixTimer != NULL) {
-		EVENTQUEUE->removeHandler(CEvent::kTimer, m_fixTimer);
-		EVENTQUEUE->deleteTimer(m_fixTimer);
+		m_events->removeHandler(CEvent::kTimer, m_fixTimer);
+		m_events->deleteTimer(m_fixTimer);
 		m_fixTimer = NULL;
 	}
 
@@ -363,6 +361,31 @@ CMSWindowsScreen::leave()
 	m_isOnScreen = false;
 	forceShowCursor();
 
+	if (getDraggingStarted()) {
+		CString& draggingDir = getDraggingFileDir();
+		size_t size = draggingDir.size();
+
+		if (!m_isPrimary) {
+			// TODO: fake these keys properly
+			fakeKeyDown(kKeyEscape, 8192, 1);
+			fakeKeyUp(1);
+
+			fakeMouseButton(kButtonLeft, false);
+
+			if (draggingDir.empty() == false) {
+				CClientApp& app = CClientApp::instance();
+				CClient* client = app.getClientPtr();
+				UInt32 fileCount = 1;
+				LOG((CLOG_DEBUG "send dragging info to server: %s", draggingDir.c_str()));
+				client->draggingInfoSending(fileCount, draggingDir, size);
+				LOG((CLOG_DEBUG "send dragging file to server"));
+				client->sendFileToServer(draggingDir.c_str());
+			}
+		}
+
+		m_draggingStarted = false;
+	}
+	
 	return true;
 }
 
@@ -402,8 +425,8 @@ CMSWindowsScreen::checkClipboards()
 	if (m_ownClipboard && !CMSWindowsClipboard::isOwnedBySynergy()) {
 		LOG((CLOG_DEBUG "clipboard changed: lost ownership and no notification received"));
 		m_ownClipboard = false;
-		sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardClipboard);
-		sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardSelection);
+		sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardClipboard);
+		sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardSelection);
 	}
 }
 
@@ -676,7 +699,7 @@ CMSWindowsScreen::getJumpZoneSize() const
 }
 
 bool
-CMSWindowsScreen::isAnyMouseButtonDown() const
+CMSWindowsScreen::isAnyMouseButtonDown(UInt32& buttonID) const
 {
 	static const char* buttonToName[] = {
 		"<invalid>",
@@ -689,6 +712,7 @@ CMSWindowsScreen::isAnyMouseButtonDown() const
 
 	for (UInt32 i = 1; i < sizeof(m_buttons) / sizeof(m_buttons[0]); ++i) {
 		if (m_buttons[i]) {
+			buttonID = i;
 			LOG((CLOG_DEBUG "locked by \"%s\"", buttonToName[i]));
 			return true;
 		}
@@ -704,28 +728,30 @@ CMSWindowsScreen::getCursorCenter(SInt32& x, SInt32& y) const
 	y = m_yCenter;
 }
 
-void
-CMSWindowsScreen::gameDeviceTimingResp(UInt16 freq)
-{
-	m_gameDevice->gameDeviceTimingResp(freq);
-}
-
-void
-CMSWindowsScreen::gameDeviceFeedback(GameDeviceID id, UInt16 m1, UInt16 m2)
-{
-	m_gameDevice->gameDeviceFeedback(id, m1, m2);
-}
-
 void
 CMSWindowsScreen::fakeMouseButton(ButtonID id, bool press)
 {
 	m_desks->fakeMouseButton(id, press);
+
+	if (id == kButtonLeft) {
+		if (press) {
+			m_buttons[kButtonLeft] = true;
+		}
+		else {
+			m_buttons[kButtonLeft] = false;
+			m_fakeDraggingStarted = false;
+			m_draggingStarted = false;
+		}
+	}
 }
 
 void
-CMSWindowsScreen::fakeMouseMove(SInt32 x, SInt32 y) const
+CMSWindowsScreen::fakeMouseMove(SInt32 x, SInt32 y)
 {
 	m_desks->fakeMouseMove(x, y);
+	if (m_buttons[kButtonLeft]) {
+		m_draggingStarted = true;
+	}
 }
 
 void
@@ -740,34 +766,6 @@ CMSWindowsScreen::fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const
 	m_desks->fakeMouseWheel(xDelta, yDelta);
 }
 
-void
-CMSWindowsScreen::fakeGameDeviceButtons(GameDeviceID id, GameDeviceButton buttons) const
-{
-	LOG((CLOG_DEBUG "fake game device buttons id=%d buttons=%d", id, buttons));
-	m_gameDevice->fakeGameDeviceButtons(id, buttons);
-}
-
-void
-CMSWindowsScreen::fakeGameDeviceSticks(GameDeviceID id, SInt16 x1, SInt16 y1, SInt16 x2, SInt16 y2) const
-{
-	LOG((CLOG_DEBUG "fake game device sticks id=%d s1=%+d,%+d s2=%+d,%+d", id, x1, y1, x2, y2));
-	m_gameDevice->fakeGameDeviceSticks(id, x1, y1, x2, y2);
-}
-
-void
-CMSWindowsScreen::fakeGameDeviceTriggers(GameDeviceID id, UInt8 t1, UInt8 t2) const
-{
-	LOG((CLOG_DEBUG "fake game device triggers id=%d t1=%d t2=%d", id, t1, t2));
-	m_gameDevice->fakeGameDeviceTriggers(id, t1, t2);
-}
-
-void
-CMSWindowsScreen::queueGameDeviceTimingReq() const
-{
-	LOG((CLOG_DEBUG "queue game device timing request"));
-	m_gameDevice->queueGameDeviceTimingReq();
-}
-
 void
 CMSWindowsScreen::updateKeys()
 {
@@ -812,6 +810,12 @@ CMSWindowsScreen::openHookLibrary(const char* name)
 	return m_hookLibraryLoader.openHookLibrary(name);
 }
 
+HINSTANCE
+CMSWindowsScreen::openShellLibrary(const char* name)
+{
+	return m_hookLibraryLoader.openShellLibrary(name);
+}
+
 void
 CMSWindowsScreen::closeHookLibrary(HINSTANCE hookLibrary) const
 {
@@ -821,6 +825,14 @@ CMSWindowsScreen::closeHookLibrary(HINSTANCE hookLibrary) const
 	}
 }
 
+void
+CMSWindowsScreen::closeShellLibrary(HINSTANCE shellLibrary) const
+{
+	if (shellLibrary != NULL) {
+		FreeLibrary(shellLibrary);
+	}
+}
+
 HCURSOR
 CMSWindowsScreen::createBlankCursor() const
 {
@@ -904,7 +916,7 @@ CMSWindowsScreen::destroyWindow(HWND hwnd) const
 void
 CMSWindowsScreen::sendEvent(CEvent::Type type, void* data)
 {
-	EVENTQUEUE->addEvent(CEvent(type, getEventTarget(), data));
+	m_events->addEvent(CEvent(type, getEventTarget(), data));
 }
 
 void
@@ -1050,7 +1062,7 @@ CMSWindowsScreen::onEvent(HWND, UINT msg,
 	case WM_ENDSESSION:
 		if (m_is95Family) {
 			if (wParam == TRUE && lParam == 0) {
-				EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+				m_events->addEvent(CEvent(CEvent::kQuit));
 			}
 			return true;
 		}
@@ -1083,13 +1095,13 @@ CMSWindowsScreen::onEvent(HWND, UINT msg,
 		case PBT_APMRESUMEAUTOMATIC:
 		case PBT_APMRESUMECRITICAL:
 		case PBT_APMRESUMESUSPEND:
-			EVENTQUEUE->addEvent(CEvent(IScreen::getResumeEvent(),
+			m_events->addEvent(CEvent(m_events->forIScreen().resume(),
 							getEventTarget(), NULL,
 							CEvent::kDeliverImmediately));
 			break;
 
 		case PBT_APMSUSPEND:
-			EVENTQUEUE->addEvent(CEvent(IScreen::getSuspendEvent(),
+			m_events->addEvent(CEvent(m_events->forIScreen().suspend(),
 							getEventTarget(), NULL,
 							CEvent::kDeliverImmediately));
 			break;
@@ -1295,14 +1307,14 @@ CMSWindowsScreen::onHotKey(WPARAM wParam, LPARAM lParam)
 			// ignore key repeats but it counts as a hot key
 			return true;
 		}
-		type = getHotKeyDownEvent();
+		type = m_events->forIPrimaryScreen().hotKeyDown();
 	}
 	else {
-		type = getHotKeyUpEvent();
+		type = m_events->forIPrimaryScreen().hotKeyUp();
 	}
 
 	// generate event
-	EVENTQUEUE->addEvent(CEvent(type, getEventTarget(),
+	m_events->addEvent(CEvent(type, getEventTarget(),
 							CHotKeyInfo::alloc(i->second)));
 
 	return true;
@@ -1319,9 +1331,16 @@ CMSWindowsScreen::onMouseButton(WPARAM wParam, LPARAM lParam)
 	if (button >= kButtonLeft && button <= kButtonExtra0 + 1) {
 		if (pressed) {
 			m_buttons[button] = true;
+			if (button == kButtonLeft) {
+				m_draggingFileDir.clear();
+				LOG((CLOG_DEBUG2 "dragging file directory is cleared"));
+			}
 		}
 		else {
 			m_buttons[button] = false;
+			if (m_draggingStarted && button == kButtonLeft) {
+				m_draggingStarted = false;
+			}
 		}
 	}
 
@@ -1331,14 +1350,14 @@ CMSWindowsScreen::onMouseButton(WPARAM wParam, LPARAM lParam)
 		if (pressed) {
 			LOG((CLOG_DEBUG1 "event: button press button=%d", button));
 			if (button != kButtonNone) {
-				sendEvent(getButtonDownEvent(),
+				sendEvent(m_events->forIPrimaryScreen().buttonDown(),
 								CButtonInfo::alloc(button, mask));
 			}
 		}
 		else {
 			LOG((CLOG_DEBUG1 "event: button release button=%d", button));
 			if (button != kButtonNone) {
-				sendEvent(getButtonUpEvent(),
+				sendEvent(m_events->forIPrimaryScreen().buttonUp(),
 								CButtonInfo::alloc(button, mask));
 			}
 		}
@@ -1380,8 +1399,12 @@ CMSWindowsScreen::onMouseMove(SInt32 mx, SInt32 my)
 		
 		// motion on primary screen
 		sendEvent(
-			getMotionOnPrimaryEvent(),
+			m_events->forIPrimaryScreen().motionOnPrimary(),
 			CMotionInfo::alloc(m_xCursor, m_yCursor));
+
+		if (m_buttons[kButtonLeft] == true && m_draggingStarted == false) {
+			m_draggingStarted = true;
+		}
 	}
 	else 
 	{
@@ -1407,7 +1430,7 @@ CMSWindowsScreen::onMouseMove(SInt32 mx, SInt32 my)
 		}
 		else {
 			// send motion
-			sendEvent(getMotionOnSecondaryEvent(), CMotionInfo::alloc(x, y));
+			sendEvent(m_events->forIPrimaryScreen().motionOnSecondary(), CMotionInfo::alloc(x, y));
 		}
 	}
 
@@ -1420,7 +1443,7 @@ CMSWindowsScreen::onMouseWheel(SInt32 xDelta, SInt32 yDelta)
 	// ignore message if posted prior to last mark change
 	if (!ignore()) {
 		LOG((CLOG_DEBUG1 "event: button wheel delta=%+d,%+d", xDelta, yDelta));
-		sendEvent(getWheelEvent(), CWheelInfo::alloc(xDelta, yDelta));
+		sendEvent(m_events->forIPrimaryScreen().wheel(), CWheelInfo::alloc(xDelta, yDelta));
 	}
 	return true;
 }
@@ -1446,7 +1469,7 @@ CMSWindowsScreen::onScreensaver(bool activated)
 		if (!m_screensaverActive &&
 			m_screensaver->checkStarted(SYNERGY_MSG_SCREEN_SAVER, FALSE, 0)) {
 			m_screensaverActive = true;
-			sendEvent(getScreensaverActivatedEvent());
+			sendEvent(m_events->forIPrimaryScreen().screensaverActivated());
 
 			// enable display power down
 			CArchMiscWindows::removeBusyState(CArchMiscWindows::kDISPLAY);
@@ -1455,7 +1478,7 @@ CMSWindowsScreen::onScreensaver(bool activated)
 	else {
 		if (m_screensaverActive) {
 			m_screensaverActive = false;
-			sendEvent(getScreensaverDeactivatedEvent());
+			sendEvent(m_events->forIPrimaryScreen().screensaverDeactivated());
 
 			// disable display power down
 			CArchMiscWindows::addBusyState(CArchMiscWindows::kDISPLAY);
@@ -1491,7 +1514,7 @@ CMSWindowsScreen::onDisplayChange()
 		}
 
 		// send new screen info
-		sendEvent(getShapeChangedEvent());
+		sendEvent(m_events->forIScreen().shapeChanged());
 
 		LOG((CLOG_DEBUG "screen shape: %d,%d %dx%d %s", m_x, m_y, m_w, m_h, m_multimon ? "(multi-monitor)" : ""));
 	}
@@ -1508,8 +1531,8 @@ CMSWindowsScreen::onClipboardChange()
 		if (m_ownClipboard) {
 			LOG((CLOG_DEBUG "clipboard changed: lost ownership"));
 			m_ownClipboard = false;
-			sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardClipboard);
-			sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardSelection);
+			sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardClipboard);
+			sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardSelection);
 		}
 	}
 	else if (!m_ownClipboard) {
@@ -1864,3 +1887,29 @@ CMSWindowsScreen::CHotKeyItem::operator<(const CHotKeyItem& x) const
 	return (m_keycode < x.m_keycode ||
 			(m_keycode == x.m_keycode && m_mask < x.m_mask));
 }
+
+void
+CMSWindowsScreen::fakeDraggingFiles(CString str)
+{
+	
+}
+
+CString&
+CMSWindowsScreen::getDraggingFileDir()
+{
+	if (m_draggingStarted) {
+		// temporarily log out dragging file directory
+		char dir[MAX_PATH];
+		m_hookLibraryLoader.m_getDraggingFileDir(dir);
+		m_draggingFileDir.clear();
+		m_draggingFileDir.append(dir);
+	}
+
+	return m_draggingFileDir;
+}
+
+const CString&
+CMSWindowsScreen::getDropTarget() const
+{
+	return m_desktopPath;
+}
