@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2002 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2002 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,14 +20,14 @@
 #define CMSWINDOWSSCREEN_H
 
 #include "CPlatformScreen.h"
-#include "CSynergyHook.h"
+#include "synwinhk.h"
 #include "CCondVar.h"
 #include "CMutex.h"
 #include "CString.h"
+#include "CMSWindowsHookLibraryLoader.h"
+
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
-#include "CMSWindowsHookLibraryLoader.h"
-#include <vector>
 
 class CEventQueueTimer;
 class CMSWindowsDesks;
@@ -34,19 +35,14 @@ class CMSWindowsKeyState;
 class CMSWindowsScreenSaver;
 class CThread;
 
-enum
-{
-	kGamePollFreqDefault = 100,
-	kGamePollFreqMin = 50,
-	kGamePollFreqMax = 200,
-	kGameCalibrationPeriod = 10000, // 10 seconds
-	kGameLagRecordMax = 10,
-};
-
 //! Implementation of IPlatformScreen for Microsoft Windows
 class CMSWindowsScreen : public CPlatformScreen {
 public:
-	CMSWindowsScreen(bool isPrimary, bool noHooks, bool gameDevice);
+	CMSWindowsScreen(
+		bool isPrimary,
+		bool noHooks,
+		bool stopOnDeskSwitch,
+		IEventQueue* events);
 	virtual ~CMSWindowsScreen();
 
 	//! @name manipulators
@@ -67,7 +63,7 @@ public:
 	/*!
 	Returns the application instance handle passed to init().
 	*/
-	static HINSTANCE	getInstance();
+	static HINSTANCE	getWindowInstance();
 
 	//@}
 
@@ -87,20 +83,14 @@ public:
 	virtual void		fakeInputBegin();
 	virtual void		fakeInputEnd();
 	virtual SInt32		getJumpZoneSize() const;
-	virtual bool		isAnyMouseButtonDown() const;
+	virtual bool		isAnyMouseButtonDown(UInt32& buttonID) const;
 	virtual void		getCursorCenter(SInt32& x, SInt32& y) const;
-	virtual void		gameDeviceTimingResp(UInt16 freq);
-	virtual void		gameDeviceFeedback(GameDeviceID id, UInt16 m1, UInt16 m2);
 
 	// ISecondaryScreen overrides
 	virtual void		fakeMouseButton(ButtonID id, bool press);
-	virtual void		fakeMouseMove(SInt32 x, SInt32 y) const;
+	virtual void		fakeMouseMove(SInt32 x, SInt32 y);
 	virtual void		fakeMouseRelativeMove(SInt32 dx, SInt32 dy) const;
 	virtual void		fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const;
-	virtual void		fakeGameDeviceButtons(GameDeviceID id, GameDeviceButton buttons) const;
-	virtual void		fakeGameDeviceSticks(GameDeviceID id, SInt16 x1, SInt16 y1, SInt16 x2, SInt16 y2) const;
-	virtual void		fakeGameDeviceTriggers(GameDeviceID id, UInt8 t1, UInt8 t2) const;
-	virtual void		queueGameDeviceTimingReq() const;
 
 	// IKeyState overrides
 	virtual void		updateKeys();
@@ -125,6 +115,10 @@ public:
 	virtual void		setOptions(const COptionsList& options);
 	virtual void		setSequenceNumber(UInt32);
 	virtual bool		isPrimary() const;
+	virtual void		fakeDraggingFiles(CString str);
+	virtual CString&	getDraggingFileDir();
+	virtual const CString&	
+						getDropTarget() const;
 
 protected:
 	// IPlatformScreen overrides
@@ -135,7 +129,9 @@ protected:
 private:
 	// initialization and shutdown operations
 	HINSTANCE			openHookLibrary(const char* name);
+	HINSTANCE			openShellLibrary(const char* name);
 	void				closeHookLibrary(HINSTANCE hookLibrary) const;
+	void				closeShellLibrary(HINSTANCE shellLibrary) const;
 	HCURSOR				createBlankCursor() const;
 	void				destroyCursor(HCURSOR cursor) const;
 	ATOM				createWindowClass() const;
@@ -145,7 +141,9 @@ private:
 	void				destroyWindow(HWND) const;
 
 	// convenience function to send events
+public: // HACK
 	void				sendEvent(CEvent::Type type, void* = NULL);
+private: // HACK
 	void				sendClipboardEvent(CEvent::Type type, ClipboardID id);
 
 	// handle message before it gets dispatched.  returns true iff
@@ -213,6 +211,9 @@ private:
 
 	// our window proc
 	static LRESULT CALLBACK wndProc(HWND, UINT, WPARAM, LPARAM);
+	
+	// save last position of mouse to compute next delta movement
+	void saveMousePosition(SInt32 x, SInt32 y);
 
 private:
 	struct CHotKeyItem {
@@ -231,7 +232,7 @@ private:
 	typedef std::vector<UInt32> HotKeyIDList;
 	typedef std::map<CHotKeyItem, UInt32> HotKeyToIDMap;
 
-	static HINSTANCE	s_instance;
+	static HINSTANCE	s_windowInstance;
 
 	// true if screen is being used as a primary screen, false otherwise
 	bool				m_isPrimary;
@@ -291,6 +292,7 @@ private:
 
 	// hook library stuff
 	HINSTANCE			m_hookLibrary;
+	HINSTANCE			m_shellLibrary;
 
 	// keyboard stuff
 	CMSWindowsKeyState*	m_keyState;
@@ -320,51 +322,16 @@ private:
 	MOUSEKEYS			m_mouseKeys;
 	MOUSEKEYS			m_oldMouseKeys;
 
-	// loads synrgyhk.dll
+	// loads synwinhk.dll
 	CMSWindowsHookLibraryLoader
 						m_hookLibraryLoader;
 
-#if GAME_DEVICE_SUPPORT
-	// game device stuff
-	CThread*			m_xInputPollThread;
-	CThread*			m_xInputTimingThread;
-	CThread*			m_xInputFeedbackThread;
-	// @todo game device state class and multiple controller support
-	WORD				m_gameButtonsLast;
-	BYTE				m_gameLeftTriggerLast;
-	BYTE				m_gameRightTriggerLast;
-	SHORT				m_gameLeftStickXLast;
-	SHORT				m_gameLeftStickYLast;
-	SHORT				m_gameRightStickXLast;
-	SHORT				m_gameRightStickYLast;
-	double				m_gameLastTimingSent;
-	bool				m_gameTimingWaiting;
-	UInt16				m_gameFakeLag;
-	UInt16				m_gameFakeLagMin;
-	UInt16				m_gamePollFreq;
-	SInt8				m_gamePollFreqAdjust;
-	UInt16				m_gameTimingStarted;
-	UInt16				m_gameTimingFirst;
-	UInt16				m_gameFakeLagLast;
-	bool				m_gameTimingCalibrated;
-	std::vector<UInt16>	m_gameFakeLagRecord;
-	bool				m_gameDevice;
-	HMODULE				m_xinputModule;
+	static CMSWindowsScreen*
+						s_screen;
+	
+	IEventQueue*		m_events;
 
-	// thread for polling xinput state.
-	void				xInputPollThread(void*);
-
-	// thread for checking queued timing requests. 
-	void				xInputTimingThread(void*);
-
-	// thread for checking pending feedback state.
-	void				xInputFeedbackThread(void*);
-#endif
-
-	static CMSWindowsScreen*	s_screen;
-
-	// save last position of mouse to compute next delta movement
-	void saveMousePosition(SInt32 x, SInt32 y);
+	CString				m_desktopPath;
 };
 
 #endif
