@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2004 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2004 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -148,7 +149,7 @@ static const CKeyEntry	s_controlKeys[] = {
 	{ kKeyKP_Multiply,	67 },
 	{ kKeyKP_Add,		69 },
 	{ kKeyKP_Divide,	75 },
-	{ kKeyKP_Subtract,	79 },
+	{ kKeyKP_Subtract,	78 },
 	{ kKeyKP_Enter,		76 },
 #endif
 
@@ -179,14 +180,15 @@ static const CKeyEntry	s_controlKeys[] = {
 // COSXKeyState
 //
 
-COSXKeyState::COSXKeyState() :
+COSXKeyState::COSXKeyState(IEventQueue* events) :
+	CKeyState(events),
 	m_deadKeyState(0)
 {
 	init();
 }
 
-COSXKeyState::COSXKeyState(IEventQueue& eventQueue, CKeyMap& keyMap) :
-	CKeyState(eventQueue, keyMap),
+COSXKeyState::COSXKeyState(IEventQueue* events, CKeyMap& keyMap) :
+	CKeyState(events, keyMap),
 	m_deadKeyState(0)
 {
 	init();
@@ -219,26 +221,23 @@ COSXKeyState::mapModifiersFromOSX(UInt32 mask) const
 {
 	LOG((CLOG_DEBUG1 "mask: %04x", mask));
 
-	// previously this used the kCGEventFlagMask* enums, which would
-	// not work, since GetCurrentKeyModifiers doesn't return a mask
-	// containing those values; instead *Key enums should be used.
 	KeyModifierMask outMask = 0;
-	if ((mask & shiftKey) != 0) {
+	if ((mask & kCGEventFlagMaskShift) != 0) {
 		outMask |= KeyModifierShift;
 	}
-	if ((mask & controlKey) != 0) {
+	if ((mask & kCGEventFlagMaskControl) != 0) {
 		outMask |= KeyModifierControl;
 	}
-	if ((mask & cmdKey) != 0) {
+	if ((mask & kCGEventFlagMaskAlternate) != 0) {
 		outMask |= KeyModifierAlt;
 	}
-	if ((mask & optionKey) != 0) {
+	if ((mask & kCGEventFlagMaskCommand) != 0) {
 		outMask |= KeyModifierSuper;
 	}
-	if ((mask & alphaLock) != 0) {
+	if ((mask & kCGEventFlagMaskAlphaShift) != 0) {
 		outMask |= KeyModifierCapsLock;
 	}
-	if ((mask & s_osxNumLock) != 0) {
+	if ((mask & kCGEventFlagMaskNumericPad) != 0) {
 		outMask |= KeyModifierNumLock;
 	}
 
