@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2004 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2004 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +32,12 @@
 #include "TMethodEventJob.h"
 #include "TMethodJob.h"
 #include "XArch.h"
+#include "COSXDragSimulator.h"
+#include "COSXPasteboardPeeker.h"
+#include "CClientApp.h"
+#include "CClient.h"
+
+#include <math.h>
 
 #include <mach-o/dyld.h>
 #include <AvailabilityMacros.h>
@@ -58,13 +65,12 @@ enum {
 // COSXScreen
 //
 
-
-
 bool					COSXScreen::s_testedForGHOM = false;
 bool					COSXScreen::s_hasGHOM	    = false;
-CEvent::Type			COSXScreen::s_confirmSleepEvent = CEvent::kUnknown;
 
-COSXScreen::COSXScreen(bool isPrimary) :
+COSXScreen::COSXScreen(IEventQueue* events, bool isPrimary, bool autoShowHideCursor) :
+	CPlatformScreen(events),
+	m_events(events),
 	MouseButtonEventMap(NumButtonIDs),
 	m_isPrimary(isPrimary),
 	m_isOnScreen(m_isPrimary),
@@ -87,16 +93,24 @@ COSXScreen::COSXScreen(bool isPrimary) :
 	m_activeModifierHotKey(0),
 	m_activeModifierHotKeyMask(0),
 	m_lastSingleClick(0),
-	m_lastDoubleClick(0)
+	m_lastDoubleClick(0),
+	m_lastSingleClickXCursor(0),
+	m_lastSingleClickYCursor(0),
+	m_autoShowHideCursor(autoShowHideCursor),
+	m_eventTapRLSR(nullptr),
+	m_eventTapPort(nullptr),
+	m_pmRootPort(0),
+	m_getDropTargetThread(NULL)
 {
 	try {
 		m_displayID   = CGMainDisplayID();
 		updateScreenShape(m_displayID, 0);
-		m_screensaver = new COSXScreenSaver(getEventTarget());
-		m_keyState	  = new COSXKeyState();
+		m_screensaver = new COSXScreenSaver(m_events, getEventTarget());
+		m_keyState	  = new COSXKeyState(m_events);
 		
+    // TODO: http://stackoverflow.com/questions/2950124/enable-access-for-assistive-device-programmatically
 		if (m_isPrimary && !AXAPIEnabled())
-			LOG((CLOG_WARN "mac accessibility api is not enabled"));
+			throw XArch("system setting not enabled: \"Enable access for assistive devices\"");
 		
 		// install display manager notification handler
 #if defined(MAC_OS_X_VERSION_10_5)
@@ -124,18 +138,19 @@ COSXScreen::COSXScreen(bool isPrimary) :
 		constructMouseButtonEventMap();
 
 		// watch for requests to sleep
-		EVENTQUEUE->adoptHandler(COSXScreen::getConfirmSleepEvent(),
+		m_events->adoptHandler(m_events->forCOSXScreen().confirmSleep(),
 								getEventTarget(),
 								new TMethodEventJob<COSXScreen>(this,
 									&COSXScreen::handleConfirmSleep));
 
 		// create thread for monitoring system power state.
+		*m_pmThreadReady = false;
 		LOG((CLOG_DEBUG "starting watchSystemPowerThread"));
 		m_pmWatchThread = new CThread(new TMethodJob<COSXScreen>
 								(this, &COSXScreen::watchSystemPowerThread));
 	}
 	catch (...) {
-		EVENTQUEUE->removeHandler(COSXScreen::getConfirmSleepEvent(),
+		m_events->removeHandler(m_events->forCOSXScreen().confirmSleep(),
 								getEventTarget());
 		if (m_switchEventHandlerRef != 0) {
 			RemoveEventHandler(m_switchEventHandlerRef);
@@ -165,19 +180,19 @@ COSXScreen::COSXScreen(bool isPrimary) :
 	}
 
 	// install event handlers
-	EVENTQUEUE->adoptHandler(CEvent::kSystem, IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(CEvent::kSystem, m_events->getSystemTarget(),
 							new TMethodEventJob<COSXScreen>(this,
 								&COSXScreen::handleSystemEvent));
 
 	// install the platform event queue
-	EVENTQUEUE->adoptBuffer(new COSXEventQueueBuffer);
+	m_events->adoptBuffer(new COSXEventQueueBuffer(m_events));
 }
 
 COSXScreen::~COSXScreen()
 {
 	disable();
-	EVENTQUEUE->adoptBuffer(NULL);
-	EVENTQUEUE->removeHandler(CEvent::kSystem, IEventQueue::getSystemTarget());
+	m_events->adoptBuffer(NULL);
+	m_events->removeHandler(CEvent::kSystem, m_events->getSystemTarget());
 
 	if (m_pmWatchThread) {
 		// make sure the thread has setup the runloop.
@@ -198,7 +213,7 @@ COSXScreen::~COSXScreen()
 	delete m_pmThreadReady;
 	delete m_pmMutex;
 
-	EVENTQUEUE->removeHandler(COSXScreen::getConfirmSleepEvent(),
+	m_events->removeHandler(m_events->forCOSXScreen().confirmSleep(),
 								getEventTarget());
 
 	RemoveEventHandler(m_switchEventHandlerRef);
@@ -298,8 +313,13 @@ COSXScreen::getJumpZoneSize() const
 }
 
 bool
-COSXScreen::isAnyMouseButtonDown() const
+COSXScreen::isAnyMouseButtonDown(UInt32& buttonID) const
 {
+	if (m_buttonState.test(0)) {
+		buttonID = kButtonLeft;
+		return true;
+	}
+
 	return (GetCurrentButtonState() != 0);
 }
 
@@ -506,8 +526,19 @@ COSXScreen::fakeMouseButton(ButtonID id, bool press)
 	pos.x = m_xCursor;
 	pos.y = m_yCursor;
 
+	// variable used to detect mouse coordinate differences between
+	// old & new mouse clicks. Used in double click detection.
+	SInt32 xDiff = m_xCursor - m_lastSingleClickXCursor;
+	SInt32 yDiff = m_yCursor - m_lastSingleClickYCursor;
+	double diff = sqrt(xDiff * xDiff + yDiff * yDiff);
+	// max sqrt(x^2 + y^2) difference allowed to double click
+	// since we don't have double click distance in NX APIs
+	// we define our own defaults.
+	const double maxDiff = sqrt(2) + 0.0001;
+
 	if (press && (id == kButtonLeft) &&
-		((ARCH->time() - m_lastSingleClick) <= clickTime)) {
+		((ARCH->time() - m_lastSingleClick) <= clickTime) &&
+		diff <= maxDiff) {
 
 		LOG((CLOG_DEBUG1 "faking mouse left double click"));
 		
@@ -543,7 +574,7 @@ COSXScreen::fakeMouseButton(ButtonID id, bool press)
 		
 		MouseButtonState state = press ? kMouseButtonDown : kMouseButtonUp;
 		
-		LOG((CLOG_DEBUG1 "faking mouse button %s", press ? "press" : "release"));
+		LOG((CLOG_DEBUG1 "faking mouse button id: %d press: %s", id, press ? "pressed" : "released"));
 		
 		MouseButtonEventMapType thisButtonMap = MouseButtonEventMap[index];
 		CGEventType type = thisButtonMap[state];
@@ -556,12 +587,72 @@ COSXScreen::fakeMouseButton(ButtonID id, bool press)
 		CFRelease(event);
 	
 		m_lastSingleClick = ARCH->time();
+		m_lastSingleClickXCursor = m_xCursor;
+		m_lastSingleClickYCursor = m_yCursor;
+	}
+	if (!press && (id == kButtonLeft)) {
+		// fake ctrl key up
+		fakeKeyUp(29);
+		
+		if (m_fakeDraggingStarted) {
+			m_getDropTargetThread = new CThread(new TMethodJob<COSXScreen>(
+				this, &COSXScreen::getDropTargetThread));
+		}
+		
+		m_draggingStarted = false;
 	}
 }
 
 void
-COSXScreen::fakeMouseMove(SInt32 x, SInt32 y) const
+COSXScreen::getDropTargetThread(void*)
 {
+#if defined(MAC_OS_X_VERSION_10_7)
+	char* cstr = NULL;
+	
+	// wait for 5 secs for the drop destinaiton string to be filled.
+	UInt32 timeout = ARCH->time() + 5;
+	
+	while (ARCH->time() < timeout) {
+		CFStringRef cfstr = getCocoaDropTarget();
+		cstr = CFStringRefToUTF8String(cfstr);
+		CFRelease(cfstr);
+		
+		if (cstr != NULL) {
+			break;
+		}
+		ARCH->sleep(.1f);
+	}
+	
+	if (cstr != NULL) {
+		LOG((CLOG_DEBUG "drop target: %s", cstr));
+		m_dropTarget = cstr;
+	}
+	else {
+		LOG((CLOG_ERR "failed to get drop target"));
+		m_dropTarget.clear();
+	}
+#else
+	LOG((CLOG_WARN "drag drop not supported"));
+#endif
+	m_fakeDraggingStarted = false;
+}
+
+void
+COSXScreen::fakeMouseMove(SInt32 x, SInt32 y)
+{
+	if (m_fakeDraggingStarted) {
+		// HACK: for some reason the drag icon
+		// does not follow the cursor unless a key
+		// is pressed (except esc key)
+		// TODO: fake this key down properly
+		fakeKeyDown(kKeyControl_L, 8194, 29);
+	}
+	
+	// index 0 means left mouse button
+	if (m_buttonState.test(0)) {
+		m_draggingStarted = true;
+	}
+	
 	// synthesize event
 	CGPoint pos;
 	pos.x = x;
@@ -625,29 +716,66 @@ void
 COSXScreen::showCursor()
 {
 	LOG((CLOG_DEBUG "showing cursor"));
-    CGDisplayShowCursor(m_displayID);
-    CFStringRef propertyString = CFStringCreateWithCString(NULL, "SetsCursorInBackground", kCFStringEncodingMacRoman);
-    CGSSetConnectionProperty(_CGSDefaultConnection(), _CGSDefaultConnection(), propertyString, kCFBooleanFalse);
-    CFRelease(propertyString);
+
+	CFStringRef propertyString = CFStringCreateWithCString(
+		NULL, "SetsCursorInBackground", kCFStringEncodingMacRoman);
+
+	CGSSetConnectionProperty(
+		_CGSDefaultConnection(), _CGSDefaultConnection(),
+		propertyString, kCFBooleanTrue);
+
+	CFRelease(propertyString);
+
+	CGError error = CGDisplayShowCursor(m_displayID);
+	if (error != kCGErrorSuccess) {
+		LOG((CLOG_ERR "failed to show cursor, error=%d", error));
+	}
+
+	// appears to fix "mouse randomly not showing" bug
+	CGAssociateMouseAndMouseCursorPosition(true);
+
+	if (!CGCursorIsVisible()) {
+		LOG((CLOG_WARN "cursor may not be visible"));
+	}
+
+	m_cursorHidden = false;
 }
 
 void
 COSXScreen::hideCursor()
 {
-    CFStringRef propertyString = CFStringCreateWithCString(NULL, "SetsCursorInBackground", kCFStringEncodingMacRoman);
-    CGSSetConnectionProperty(_CGSDefaultConnection(), _CGSDefaultConnection(), propertyString, kCFBooleanTrue);
-    CFRelease(propertyString);
-
 	LOG((CLOG_DEBUG "hiding cursor"));
-    CGDisplayHideCursor(m_displayID);
+
+	CFStringRef propertyString = CFStringCreateWithCString(
+		NULL, "SetsCursorInBackground", kCFStringEncodingMacRoman);
+
+	CGSSetConnectionProperty(
+		_CGSDefaultConnection(), _CGSDefaultConnection(),
+		propertyString, kCFBooleanTrue);
+
+	CFRelease(propertyString);
+
+	CGError error = CGDisplayHideCursor(m_displayID);
+	if (error != kCGErrorSuccess) {
+		LOG((CLOG_ERR "failed to hide cursor, error=%d", error));
+	}
+
+	// appears to fix "mouse randomly not hiding" bug
+	CGAssociateMouseAndMouseCursorPosition(true);
+
+	if (CGCursorIsVisible()) {
+		LOG((CLOG_WARN "cursor may be still visible"));
+	}
+
+	m_cursorHidden = true;
 }
 
 void
 COSXScreen::enable()
 {
 	// watch the clipboard
-	m_clipboardTimer = EVENTQUEUE->newTimer(1.0, NULL);
-	EVENTQUEUE->adoptHandler(CEvent::kTimer, m_clipboardTimer,
+	m_clipboardTimer = m_events->newTimer(1.0, NULL);
+	m_events->adoptHandler(CEvent::kTimer, m_clipboardTimer,
 							new TMethodEventJob<COSXScreen>(this,
 								&COSXScreen::handleClipboardCheck));
 
@@ -655,7 +783,7 @@ COSXScreen::enable()
 		// FIXME -- start watching jump zones
 		
 		// kCGEventTapOptionDefault = 0x00000000 (Missing in 10.4, so specified literally)
-		m_eventTapPort=CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, 0, 
+		m_eventTapPort = CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, 0,
 										kCGEventMaskForAllEvents, 
 										handleCGInputEvent, 
 										this);
@@ -663,10 +791,8 @@ COSXScreen::enable()
 	else {
 		// FIXME -- prevent system from entering power save mode
 
-		// hide cursor
-		if (!m_cursorHidden) {
-	                hideCursor();
-			m_cursorHidden = true;
+		if (m_autoShowHideCursor) {
+			hideCursor();
 		}
 
 		// warp the mouse to the cursor center
@@ -675,39 +801,41 @@ COSXScreen::enable()
                 // there may be a better way to do this, but we register an event handler even if we're
                 // not on the primary display (acting as a client). This way, if a local event comes in
                 // (either keyboard or mouse), we can make sure to show the cursor if we've hidden it. 
-		m_eventTapPort=CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, 0, 
+		m_eventTapPort = CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, 0,
 										kCGEventMaskForAllEvents, 
 										handleCGInputEventSecondary, 
 										this);
 	}
-	if(!m_eventTapPort) {
+
+	if (!m_eventTapPort) {
 		LOG((CLOG_ERR "failed to create quartz event tap"));
 	}
-	m_eventTapRLSR=CFMachPortCreateRunLoopSource(kCFAllocatorDefault, m_eventTapPort, 0);
-	if(!m_eventTapRLSR) {
+
+	m_eventTapRLSR = CFMachPortCreateRunLoopSource(kCFAllocatorDefault, m_eventTapPort, 0);
+	if (!m_eventTapRLSR) {
 		LOG((CLOG_ERR "failed to create a CFRunLoopSourceRef for the quartz event tap"));
 	}
+
 	CFRunLoopAddSource(CFRunLoopGetCurrent(), m_eventTapRLSR, kCFRunLoopDefaultMode);
 }
 
 void
 COSXScreen::disable()
 {
-        // show cursor if hidden
-        if (m_cursorHidden) {
-                showCursor();
-		m_cursorHidden = false;
+	if (m_autoShowHideCursor) {
+		showCursor();
 	}
     
-        // FIXME -- stop watching jump zones, stop capturing input
+	// FIXME -- stop watching jump zones, stop capturing input
 	
-	if(m_eventTapRLSR) {
+	if (m_eventTapRLSR) {
 		CFRelease(m_eventTapRLSR);
-		m_eventTapRLSR=NULL;
-	}		
-	if(m_eventTapPort) {
+		m_eventTapRLSR = nullptr;
+	}
+
+	if (m_eventTapPort) {
 		CFRelease(m_eventTapPort);
-		m_eventTapPort=NULL;
+		m_eventTapPort = nullptr;
 	}
 	// FIXME -- allow system to enter power saving mode
 
@@ -717,8 +845,8 @@ COSXScreen::disable()
 
 	// uninstall clipboard timer
 	if (m_clipboardTimer != NULL) {
-		EVENTQUEUE->removeHandler(CEvent::kTimer, m_clipboardTimer);
-		EVENTQUEUE->deleteTimer(m_clipboardTimer);
+		m_events->removeHandler(CEvent::kTimer, m_clipboardTimer);
+		m_events->deleteTimer(m_clipboardTimer);
 		m_clipboardTimer = NULL;
 	}
 
@@ -728,11 +856,8 @@ COSXScreen::disable()
 void
 COSXScreen::enter()
 {
-    // show cursor
-	if (m_cursorHidden) {
-                showCursor();
-		m_cursorHidden = false;
-	}
+	showCursor();
+
 	if (m_isPrimary) {
 		CGSetLocalEventsSuppressionInterval(0.0);
 		
@@ -743,6 +868,17 @@ COSXScreen::enter()
 		// reset buttons
 		m_buttonState.reset();
 
+		// patch by Yutaka Tsutano
+		// wakes the client screen
+		// http://synergy-foss.org/spit/issues/details/3287#c12
+		io_registry_entry_t entry = IORegistryEntryFromPath(
+			kIOMasterPortDefault,
+			"IOService:/IOResources/IODisplayWrangler");
+		if (entry != MACH_PORT_NULL) {
+			IORegistryEntrySetCFProperty(entry, CFSTR("IORequestIdle"), kCFBooleanFalse);
+			IOObjectRelease(entry);
+		}
+
 		// avoid suppression of local hardware events
 		// stkamp@users.sourceforge.net
 		CGSetLocalEventsFilterDuringSupressionState(
@@ -761,15 +897,37 @@ COSXScreen::enter()
 bool
 COSXScreen::leave()
 {
-    // hide cursor
-	if (!m_cursorHidden) {
-        hideCursor();
-		m_cursorHidden = true;
-	}
+    hideCursor();
     
+	if (getDraggingStarted()) {
+		CString& fileList = getDraggingFileDir();
+		size_t size = fileList.size();
+		
+		if (!m_isPrimary) {
+			// fake esc key down and up
+			fakeKeyDown(kKeyEscape, 8192, 1);
+			fakeKeyUp(1);
+			
+			fakeMouseButton(kButtonLeft, false);
+			
+			// fake ctrl key up
+			fakeKeyUp(29);
+			
+			if (fileList.empty() == false) {
+				CClientApp& app = CClientApp::instance();
+				CClient* client = app.getClientPtr();
+				UInt32 fileCount = 1;
+				client->draggingInfoSending(fileCount, fileList, size);
+				LOG((CLOG_DEBUG "send dragging file to server"));
+				client->sendFileToServer(fileList.c_str());
+			}
+		}
+		m_draggingStarted = false;
+	}
+	
 	if (m_isPrimary) {
 		// warp to center
-		warpCursor(m_xCenter, m_yCenter);
+		//warpCursor(m_xCenter, m_yCenter);
 		
 		// This used to be necessary to get smooth mouse motion on other screens,
 		// but now is just to avoid a hesitating cursor when transitioning to
@@ -781,7 +939,7 @@ COSXScreen::leave()
 	}
 	else {
 		// warp the mouse to the cursor center
-		fakeMouseMove(m_xCenter, m_yCenter);
+		//fakeMouseMove(m_xCenter, m_yCenter);
 
 		// FIXME -- prepare to show cursor if it moves
 
@@ -811,8 +969,8 @@ COSXScreen::checkClipboards()
 	LOG((CLOG_DEBUG2 "checking clipboard"));
 	if (m_pasteboard.synchronize()) {
 		LOG((CLOG_DEBUG "clipboard changed"));
-		sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardClipboard);
-		sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardSelection);
+		sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardClipboard);
+		sendClipboardEvent(m_events->forIScreen().clipboardGrabbed(), kClipboardSelection);
 	}
 }
 
@@ -871,7 +1029,7 @@ COSXScreen::isPrimary() const
 void
 COSXScreen::sendEvent(CEvent::Type type, void* data) const
 {
-	EVENTQUEUE->addEvent(CEvent(type, getEventTarget(), data));
+	m_events->addEvent(CEvent(type, getEventTarget(), data));
 }
 
 void
@@ -985,8 +1143,11 @@ COSXScreen::onMouseMove(SInt32 mx, SInt32 my)
 
 	if (m_isOnScreen) {
 		// motion on primary screen
-		sendEvent(getMotionOnPrimaryEvent(),
+		sendEvent(m_events->forIPrimaryScreen().motionOnPrimary(),
 							CMotionInfo::alloc(m_xCursor, m_yCursor));
+		if (m_buttonState.test(0)) {
+			m_draggingStarted = true;
+		}
 	}
 	else {
 		// motion on secondary screen.  warp mouse back to
@@ -1007,7 +1168,7 @@ COSXScreen::onMouseMove(SInt32 mx, SInt32 my)
 		}
 		else {
 			// send motion
-			sendEvent(getMotionOnSecondaryEvent(), CMotionInfo::alloc(x, y));
+			sendEvent(m_events->forIPrimaryScreen().motionOnSecondary(), CMotionInfo::alloc(x, y));
 		}
 	}
 
@@ -1024,14 +1185,14 @@ COSXScreen::onMouseButton(bool pressed, UInt16 macButton)
 		LOG((CLOG_DEBUG1 "event: button press button=%d", button));
 		if (button != kButtonNone) {
 			KeyModifierMask mask = m_keyState->getActiveModifiers();
-			sendEvent(getButtonDownEvent(), CButtonInfo::alloc(button, mask));
+			sendEvent(m_events->forIPrimaryScreen().buttonDown(), CButtonInfo::alloc(button, mask));
 		}
 	}
 	else {
 		LOG((CLOG_DEBUG1 "event: button release button=%d", button));
 		if (button != kButtonNone) {
 			KeyModifierMask mask = m_keyState->getActiveModifiers();
-			sendEvent(getButtonUpEvent(), CButtonInfo::alloc(button, mask));
+			sendEvent(m_events->forIPrimaryScreen().buttonUp(), CButtonInfo::alloc(button, mask));
 		}
 	}
 
@@ -1050,6 +1211,23 @@ COSXScreen::onMouseButton(bool pressed, UInt16 macButton)
 			}
 		}
 	}
+	
+	if (macButton == kButtonLeft) {
+		MouseButtonState state = pressed ? kMouseButtonDown : kMouseButtonUp;
+		m_buttonState.set(kButtonLeft - 1, state);
+		if (pressed) {
+			m_draggingFileDir.clear();
+			LOG((CLOG_DEBUG2 "dragging file directory is cleared"));
+		}
+		else {
+			if (m_fakeDraggingStarted) {
+				m_getDropTargetThread = new CThread(new TMethodJob<COSXScreen>(
+																			   this, &COSXScreen::getDropTargetThread));
+			}
+			
+			m_draggingStarted = false;
+		}
+	}
 
 	return true;
 }
@@ -1058,7 +1236,7 @@ bool
 COSXScreen::onMouseWheel(SInt32 xDelta, SInt32 yDelta) const
 {
 	LOG((CLOG_DEBUG1 "event: button wheel delta=%+d,%+d", xDelta, yDelta));
-	sendEvent(getWheelEvent(), CWheelInfo::alloc(xDelta, yDelta));
+	sendEvent(m_events->forIPrimaryScreen().wheel(), CWheelInfo::alloc(xDelta, yDelta));
 	return true;
 }
 
@@ -1098,7 +1276,7 @@ COSXScreen::onDisplayChange()
 		}
 
 		// send new screen info
-		sendEvent(getShapeChangedEvent());
+		sendEvent(m_events->forIPrimaryScreen().shapeChanged());
 	}
 
 	return true;
@@ -1150,7 +1328,7 @@ COSXScreen::onKey(CGEventRef event)
 			if (m_modifierHotKeys.count(newMask) > 0) {
 				m_activeModifierHotKey     = m_modifierHotKeys[newMask];
 				m_activeModifierHotKeyMask = newMask;
-				EVENTQUEUE->addEvent(CEvent(getHotKeyDownEvent(),
+				m_events->addEvent(CEvent(m_events->forIPrimaryScreen().hotKeyDown(),
 								getEventTarget(),
 								CHotKeyInfo::alloc(m_activeModifierHotKey)));
 			}
@@ -1161,7 +1339,7 @@ COSXScreen::onKey(CGEventRef event)
 		else if (m_activeModifierHotKey != 0) {
 			KeyModifierMask mask = (newMask & m_activeModifierHotKeyMask);
 			if (mask != m_activeModifierHotKeyMask) {
-				EVENTQUEUE->addEvent(CEvent(getHotKeyUpEvent(),
+				m_events->addEvent(CEvent(m_events->forIPrimaryScreen().hotKeyUp(),
 								getEventTarget(),
 								CHotKeyInfo::alloc(m_activeModifierHotKey)));
 				m_activeModifierHotKey     = 0;
@@ -1188,16 +1366,16 @@ COSXScreen::onKey(CGEventRef event)
 			CEvent::Type type;
 			//UInt32 eventKind = GetEventKind(event);
 			if (eventKind == kCGEventKeyDown) {
-				type = getHotKeyDownEvent();
+				type = m_events->forIPrimaryScreen().hotKeyDown();
 			}
 			else if (eventKind == kCGEventKeyUp) {
-				type = getHotKeyUpEvent();
+				type = m_events->forIPrimaryScreen().hotKeyUp();
 			}
 			else {
 				return false;
 			}
 	
-			EVENTQUEUE->addEvent(CEvent(type, getEventTarget(),
+			m_events->addEvent(CEvent(type, getEventTarget(),
 										CHotKeyInfo::alloc(id)));
 		
 			return true;
@@ -1261,16 +1439,16 @@ COSXScreen::onHotKey(EventRef event) const
 	CEvent::Type type;
 	UInt32 eventKind = GetEventKind(event);
 	if (eventKind == kEventHotKeyPressed) {
-		type = getHotKeyDownEvent();
+		type = m_events->forIPrimaryScreen().hotKeyDown();
 	}
 	else if (eventKind == kEventHotKeyReleased) {
-		type = getHotKeyUpEvent();
+		type = m_events->forIPrimaryScreen().hotKeyUp();
 	}
 	else {
 		return false;
 	}
 
-	EVENTQUEUE->addEvent(CEvent(type, getEventTarget(),
+	m_events->addEvent(CEvent(type, getEventTarget(),
 								CHotKeyInfo::alloc(id)));
 
 	return true;
@@ -1345,19 +1523,19 @@ COSXScreen::getScrollSpeedFactor() const
 void
 COSXScreen::enableDragTimer(bool enable)
 {
-  UInt32 modifiers;
-  MouseTrackingResult res; 
+	UInt32 modifiers;
+	MouseTrackingResult res;
 
 	if (enable && m_dragTimer == NULL) {
-		m_dragTimer = EVENTQUEUE->newTimer(0.01, NULL);
-		EVENTQUEUE->adoptHandler(CEvent::kTimer, m_dragTimer,
+		m_dragTimer = m_events->newTimer(0.01, NULL);
+		m_events->adoptHandler(CEvent::kTimer, m_dragTimer,
 							new TMethodEventJob<COSXScreen>(this,
 								&COSXScreen::handleDrag));
 		TrackMouseLocationWithOptions(NULL, 0, 0, &m_dragLastPoint, &modifiers, &res);
 	}
 	else if (!enable && m_dragTimer != NULL) {
-		EVENTQUEUE->removeHandler(CEvent::kTimer, m_dragTimer);
-		EVENTQUEUE->deleteTimer(m_dragTimer);
+		m_events->removeHandler(CEvent::kTimer, m_dragTimer);
+		m_events->deleteTimer(m_dragTimer);
 		m_dragTimer = NULL;
 	}
 }
@@ -1366,8 +1544,8 @@ void
 COSXScreen::handleDrag(const CEvent&, void*)
 {
 	Point p;
-  UInt32 modifiers;
-  MouseTrackingResult res; 
+	UInt32 modifiers;
+	MouseTrackingResult res; 
 
 	TrackMouseLocationWithOptions(NULL, 0, 0, &p, &modifiers, &res);
 
@@ -1443,9 +1621,9 @@ COSXScreen::updateScreenShape()
 
 	delete[] displays;
 	// We want to notify the peer screen whether we are primary screen or not
-	sendEvent(getShapeChangedEvent());
+	sendEvent(m_events->forIScreen().shapeChanged());
 
-	LOG((CLOG_DEBUG "screen shape: center=%d,%d size=%dx%d on %u %s (%s)",
+	LOG((CLOG_DEBUG "screen shape: center=%d,%d size=%dx%d on %u %s",
          m_x, m_y, m_w, m_h, displayCount,
          (displayCount == 1) ? "display" : "displays"));
 }
@@ -1467,15 +1645,16 @@ COSXScreen::userSwitchCallback(EventHandlerCallRef nextHandler,
 {
 	COSXScreen* screen = (COSXScreen*)inUserData;
 	UInt32 kind        = GetEventKind(theEvent);
+	IEventQueue* events = screen->getEvents();
 
 	if (kind == kEventSystemUserSessionDeactivated) {
 		LOG((CLOG_DEBUG "user session deactivated"));
-		EVENTQUEUE->addEvent(CEvent(IScreen::getSuspendEvent(),
+		events->addEvent(CEvent(events->forIScreen().suspend(),
 									screen->getEventTarget()));
 	}
 	else if (kind == kEventSystemUserSessionActivated) {
 		LOG((CLOG_DEBUG "user session activated"));
-		EVENTQUEUE->addEvent(CEvent(IScreen::getResumeEvent(),
+		events->addEvent(CEvent(events->forIScreen().resume(),
 									screen->getEventTarget()));
 	}
 	return (CallNextEventHandler(nextHandler, theEvent));
@@ -1526,8 +1705,13 @@ COSXScreen::watchSystemPowerThread(void*)
 		return;
 	}
 
-	// start the run loop
 	LOG((CLOG_DEBUG "started watchSystemPowerThread"));
+    
+	// HACK: sleep, this seem to stop synergy from freezing.
+	ARCH->sleep(1);
+    
+	// start the run loop
+	LOG((CLOG_DEBUG "starting carbon loop"));
 	CFRunLoopRun();
 	
 	// cleanup
@@ -1560,14 +1744,14 @@ COSXScreen::handlePowerChangeRequest(natural_t messageType, void* messageArg)
 		// COSXScreen has to handle this in the main thread so we have to
 		// queue a confirm sleep event here.  we actually don't allow the
 		// system to sleep until the event is handled.
-		EVENTQUEUE->addEvent(CEvent(COSXScreen::getConfirmSleepEvent(),
+		m_events->addEvent(CEvent(m_events->forCOSXScreen().confirmSleep(),
 								getEventTarget(), messageArg,
 								CEvent::kDontFreeData));
 		return;
 			
 	case kIOMessageSystemHasPoweredOn:
 		LOG((CLOG_DEBUG "system wakeup"));
-		EVENTQUEUE->addEvent(CEvent(IScreen::getResumeEvent(),
+		m_events->addEvent(CEvent(m_events->forIScreen().resume(),
 								getEventTarget()));
 		break;
 
@@ -1581,13 +1765,6 @@ COSXScreen::handlePowerChangeRequest(natural_t messageType, void* messageArg)
 	}
 }
 
-CEvent::Type
-COSXScreen::getConfirmSleepEvent()
-{
-	return EVENTQUEUE->registerTypeOnce(s_confirmSleepEvent,
-									"COSXScreen::confirmSleep");
-}
-
 void
 COSXScreen::handleConfirmSleep(const CEvent& event, void*)
 {
@@ -1596,7 +1773,7 @@ COSXScreen::handleConfirmSleep(const CEvent& event, void*)
 		CLock lock(m_pmMutex);
 		if (m_pmRootPort != 0) {
 			// deliver suspend event immediately.
-			EVENTQUEUE->addEvent(CEvent(IScreen::getSuspendEvent(),
+			m_events->addEvent(CEvent(m_events->forIScreen().suspend(),
 									getEventTarget(), NULL, 
 									CEvent::kDeliverImmediately));
 	
@@ -1741,32 +1918,32 @@ COSXScreen::CHotKeyItem::operator<(const CHotKeyItem& x) const
 			(m_keycode == x.m_keycode && m_mask < x.m_mask));
 }
 
-// Quartz event tap support for the secondary display. This make sure that we 
-// will show the cursor if a local event comes in while synergy has the cursor off the screen.
+// Quartz event tap support for the secondary display. This makes sure that we
+// will show the cursor if a local event comes in while synergy has the cursor
+// off the screen.
 CGEventRef
-COSXScreen::handleCGInputEventSecondary(CGEventTapProxy proxy,
-							   CGEventType type,
-							   CGEventRef event,
-							   void* refcon)
+COSXScreen::handleCGInputEventSecondary(
+	CGEventTapProxy proxy,
+	CGEventType type,
+	CGEventRef event,
+	void* refcon)
 {
-       COSXScreen* screen = (COSXScreen*)refcon;
-       if (screen->m_cursorHidden) {
-                CGPoint pos;
-                bool showCursor = true;
-                if (type == kCGEventMouseMoved) {
-                        pos = CGEventGetLocation(event);
-                        if (pos.x == screen->m_xCenter && pos.y == screen->m_yCenter) {
-                                showCursor = false;
-                        }
-                }
-                if (showCursor) {
-                        LOG((CLOG_DEBUG "Trying to show cursor from local event. (type = %d)", type));
-                        screen->showCursor();
-                        screen->m_cursorHidden = false;
-                }
-        }
-        LOG((CLOG_DEBUG2 "Local event? (type = %d)", type));
-        return event;
+	// this fix is really screwing with the correct show/hide behavior. it
+	// should be tested better before reintroducing.
+	return event;
+
+	COSXScreen* screen = (COSXScreen*)refcon;
+	if (screen->m_cursorHidden && type == kCGEventMouseMoved) {
+
+		CGPoint pos = CGEventGetLocation(event);
+		if (pos.x != screen->m_xCenter || pos.y != screen->m_yCenter) {
+
+			LOG((CLOG_DEBUG "show cursor on secondary, type=%d pos=%d,%d",
+					type, pos.x, pos.y));
+			screen->showCursor();
+		}
+	}
+	return event;
 }
 
 // Quartz event tap support
@@ -1779,15 +1956,6 @@ COSXScreen::handleCGInputEvent(CGEventTapProxy proxy,
 	COSXScreen* screen = (COSXScreen*)refcon;
 	CGPoint pos;
 
-	// Patch by Perceptum to ignore what seems to be an "Out of Bounds" 
-	// event that is emitted by touching the trackpad on a MacBook
-	// Suspicion that it is related to the new Gestures trackpad
-	// bryan@perceptum.biz
-	if(type > NX_LASTEVENT) {
-		LOG((CLOG_NOTE "Ignoring Out of Bounds Quartz Event type: 0x%02x [max 0x%02x]", type, NX_LASTEVENT));
-		return NULL;
-	}
-
 	switch(type) {
 		case kCGEventLeftMouseDown:
 		case kCGEventRightMouseDown:
@@ -1799,10 +1967,10 @@ COSXScreen::handleCGInputEvent(CGEventTapProxy proxy,
 		case kCGEventOtherMouseUp:
 			screen->onMouseButton(false, CGEventGetIntegerValueField(event, kCGMouseEventButtonNumber) + 1);
 			break;
-		case kCGEventMouseMoved:
 		case kCGEventLeftMouseDragged:
 		case kCGEventRightMouseDragged:
 		case kCGEventOtherMouseDragged:
+		case kCGEventMouseMoved:
 			pos = CGEventGetLocation(event);
 			screen->onMouseMove(pos.x, pos.y);
 			
@@ -1893,3 +2061,55 @@ COSXScreen::CMouseButtonState::getFirstButtonDown() const
 	}
 	return -1;
 }
+
+char*
+COSXScreen::CFStringRefToUTF8String(CFStringRef aString)
+{
+	if (aString == NULL) {
+		return NULL;
+	}
+	
+	CFIndex length = CFStringGetLength(aString);
+	CFIndex maxSize = CFStringGetMaximumSizeForEncoding(
+		length,
+		kCFStringEncodingUTF8);
+	char* buffer = (char*)malloc(maxSize);
+	if (CFStringGetCString(aString, buffer, maxSize, kCFStringEncodingUTF8)) {
+		return buffer;
+	}
+	return NULL;
+}
+
+void
+COSXScreen::fakeDraggingFiles(CString str)
+{
+	if (CApp::instance().argsBase().m_enableDragDrop) {
+		m_fakeDraggingStarted = true;
+#if defined(MAC_OS_X_VERSION_10_7)
+		// TODO: use real file extension
+		fakeDragging("txt", 3, m_xCursor, m_yCursor);
+#else
+		LOG((CLOG_WARN "drag drop not supported"));
+#endif
+	}
+}
+
+CString&
+COSXScreen::getDraggingFileDir()
+{
+	if (m_draggingStarted) {
+		CFStringRef dragInfo = getDraggedFileURL();
+		char* info = NULL;
+		info = CFStringRefToUTF8String(dragInfo);
+		if (info == NULL) {
+			m_draggingFileDir.clear();
+		}
+		else {
+			LOG((CLOG_DEBUG "drag info: %s", info));
+			CFRelease(dragInfo);
+			CString fileList(info);
+			m_draggingFileDir = fileList;
+		}
+	}
+	return m_draggingFileDir;
+}
