@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2002 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2002 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,15 +45,21 @@
 #include <ApplicationServices/ApplicationServices.h>
 #endif
 
+#if defined(__APPLE__)
+#include "COSXDragSimulator.h"
+#endif
+
 CApp* CApp::s_instance = nullptr;
 
-CApp::CApp(CreateTaskBarReceiverFunc createTaskBarReceiver, CArgsBase* args) :
-m_createTaskBarReceiver(createTaskBarReceiver),
-m_args(args),
-m_bye(&exit),
-m_taskBarReceiver(NULL),
-m_suspended(false),
-m_ipcClient(nullptr)
+CApp::CApp(IEventQueue* events, CreateTaskBarReceiverFunc createTaskBarReceiver, CArgsBase* args) :
+	m_events(events),
+	m_createTaskBarReceiver(createTaskBarReceiver),
+	m_args(args),
+	m_bye(&exit),
+	m_taskBarReceiver(NULL),
+	m_suspended(false),
+	m_ipcClient(nullptr),
+	m_appUtil(events)
 {
 	assert(s_instance == nullptr);
 	s_instance = this;
@@ -152,11 +159,26 @@ CApp::parseArg(const int& argc, const char* const* argv, int& i)
 		argsBase().m_enableIpc = true;
 	}
 
-#if VNC_SUPPORT
-	else if (isArg(i, argc, argv, NULL, "--vnc")) {
-		argsBase().m_enableVnc = true;
+	else if (isArg(i, argc, argv, NULL, "--server")) {
+		// HACK: stop error happening when using portable (synergyp) 
 	}
+
+	else if (isArg(i, argc, argv, NULL, "--client")) {
+		// HACK: stop error happening when using portable (synergyp) 
+	}
+
+	else if (isArg(i, argc, argv, NULL, "--crypto-pass")) {
+		argsBase().m_crypto.m_pass = argv[++i];
+		argsBase().m_crypto.setMode("cfb");
+	}
+
+	else if (isArg(i, argc, argv, NULL, "--enable-drag-drop")) {
+#ifdef WINAPI_XWINDOWS
+		std::cerr << "Option not supported on Linux: " << argv[i] << std::endl;
+		m_bye(kExitArgs);
 #endif
+		argsBase().m_enableDragDrop = true;
+	}
 
 	else {
 		// option not supported here
@@ -243,18 +265,7 @@ CApp::version()
 
 int
 CApp::run(int argc, char** argv)
-{
-#if SYSAPI_WIN32
-	// record window instance for tray icon, etc
-	CArchMiscWindows::setInstanceWin32(GetModuleHandle(NULL));
-#endif
-
-	CArch arch;
-	arch.init();
-
-	CLog log;
-	CEventQueue events;
-
+{	
 #if MAC_OS_X_VERSION_10_7
 	// dock hide only supported on lion :(
 	ProcessSerialNumber psn = { 0, kCurrentProcess };
@@ -348,18 +359,18 @@ CApp::initApp(int argc, const char** argv)
 
 		// make the task bar receiver.  the user can control this app
 		// through the task bar.
-		m_taskBarReceiver = m_createTaskBarReceiver(logBuffer);
+		m_taskBarReceiver = m_createTaskBarReceiver(logBuffer, m_events);
 	}
 }
 
 void
 CApp::initIpcClient()
 {
-	m_ipcClient = new CIpcClient();
+	m_ipcClient = new CIpcClient(m_events, m_socketMultiplexer);
 	m_ipcClient->connect();
 
-	EVENTQUEUE->adoptHandler(
-		CIpcClient::getMessageReceivedEvent(), m_ipcClient,
+	m_events->adoptHandler(
+		m_events->forCIpcClient().messageReceived(), m_ipcClient,
 		new TMethodEventJob<CApp>(this, &CApp::handleIpcMessage));
 }
 
@@ -367,7 +378,7 @@ void
 CApp::cleanupIpcClient()
 {
 	m_ipcClient->disconnect();
-	EVENTQUEUE->removeHandler(CIpcClient::getMessageReceivedEvent(), m_ipcClient);
+	m_events->removeHandler(m_events->forCIpcClient().messageReceived(), m_ipcClient);
 	delete m_ipcClient;
 }
 
@@ -377,6 +388,19 @@ CApp::handleIpcMessage(const CEvent& e, void*)
 	CIpcMessage* m = static_cast<CIpcMessage*>(e.getDataObject());
 	if (m->type() == kIpcShutdown) {
 		LOG((CLOG_INFO "got ipc shutdown message"));
-		EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
-	}
+		m_events->addEvent(CEvent(CEvent::kQuit));
+    }
+}
+
+void
+CApp::runEventsLoop(void*)
+{
+	m_events->cacheCurrentEventQueueRef();
+	m_events->loop();
+	
+#if defined(MAC_OS_X_VERSION_10_7)
+	
+	stopCocoaLoop();
+	
+#endif
 }
