@@ -1,6 +1,7 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2002 Chris Schoeneman, Nick Bolton, Sorin Sbarnea
+ * Copyright (C) 2012 Bolton Software Ltd.
+ * Copyright (C) 2002 Chris Schoeneman
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,7 +34,6 @@
 #include "LogOutputters.h"
 #include "CFunctionEventJob.h"
 #include "TMethodJob.h"
-#include "CVncClient.h"
 
 #if SYSAPI_WIN32
 #include "CArchMiscWindows.h"
@@ -47,25 +47,28 @@
 #include "COSXScreen.h"
 #endif
 
+#if defined(__APPLE__)
+#include "COSXDragSimulator.h"
+#endif
+
 #include <iostream>
 #include <stdio.h>
 #include <fstream>
 #include "XScreen.h"
 #include "CTCPSocketFactory.h"
 
-CEvent::Type CServerApp::s_reloadConfigEvent = CEvent::kUnknown;
+//
+// CServerApp
+//
 
-CServerApp::CServerApp(CreateTaskBarReceiverFunc createTaskBarReceiver) :
-CApp(createTaskBarReceiver, new CArgs()),
-s_server(NULL),
-s_forceReconnectEvent(CEvent::kUnknown),
-s_resetServerEvent(CEvent::kUnknown),
-s_serverState(kUninitialized),
-s_serverScreen(NULL),
-s_primaryClient(NULL),
-s_listener(NULL),
-s_timer(NULL),
-m_vncClient(NULL)
+CServerApp::CServerApp(IEventQueue* events, CreateTaskBarReceiverFunc createTaskBarReceiver) :
+	CApp(events, createTaskBarReceiver, new CArgs()),
+	s_server(NULL),
+	s_serverState(kUninitialized),
+	s_serverScreen(NULL),
+	s_primaryClient(NULL),
+	s_listener(NULL),
+	s_timer(NULL)
 {
 }
 
@@ -143,6 +146,10 @@ CServerApp::parseArgs(int argc, const char* const* argv)
 
 	// identify system
 	LOG((CLOG_INFO "%s Server on %s %s", kAppVersion, ARCH->getOSName().c_str(), ARCH->getPlatformName().c_str()));
+	
+	if (args().m_enableDragDrop) {
+		LOG((CLOG_INFO "drag and drop enabled"));
+	}
 
 	loggingFilterWarning();
 }
@@ -197,14 +204,15 @@ CServerApp::help()
 		ARCH->concatPath(ARCH->getSystemDirectory(), SYS_CONFIG_NAME).c_str()
 	);
 
-	std::cout << buffer << std::endl;
+	LOG((CLOG_PRINT "%s", buffer));
 }
 
 void
 CServerApp::reloadSignalHandler(CArch::ESignal, void*)
 {
-	EVENTQUEUE->addEvent(CEvent(getReloadConfigEvent(),
-		IEventQueue::getSystemTarget()));
+	IEventQueue* events = CApp::instance().getEvents();
+	events->addEvent(CEvent(events->forCServerApp().reloadConfig(),
+		events->getSystemTarget()));
 }
 
 void
@@ -289,12 +297,6 @@ CServerApp::loadConfig(const CString& pathname)
 	return false;
 }
 
-CEvent::Type 
-CServerApp::getReloadConfigEvent()
-{
-	return EVENTQUEUE->registerTypeOnce(s_reloadConfigEvent, "reloadConfig");
-}
-
 void 
 CServerApp::forceReconnect(const CEvent&, void*)
 {
@@ -303,18 +305,6 @@ CServerApp::forceReconnect(const CEvent&, void*)
 	}
 }
 
-CEvent::Type 
-CServerApp::getForceReconnectEvent()
-{
-	return EVENTQUEUE->registerTypeOnce(s_forceReconnectEvent, "forceReconnect");
-}
-
-CEvent::Type
-CServerApp::getResetServerEvent()
-{
-	return EVENTQUEUE->registerTypeOnce(s_resetServerEvent, "resetServer");
-}
-
 void 
 CServerApp::handleClientConnected(const CEvent&, void* vlistener)
 {
@@ -323,20 +313,13 @@ CServerApp::handleClientConnected(const CEvent&, void* vlistener)
 	if (client != NULL) {
 		s_server->adoptClient(client);
 		updateStatus();
-
-		if (args().m_enableVnc) {
-			// TODO: figure out client IP address from name.
-			CVncClient* vncClient = new CVncClient("192.168.0.13", client->getName());
-			vncClient->start();
-			m_vncClients.insert(std::pair<CString, CVncClient*>(client->getName(), vncClient));
-		}
 	}
 }
 
 void
 CServerApp::handleClientsDisconnected(const CEvent&, void*)
 {
-	EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+	m_events->addEvent(CEvent(CEvent::kQuit));
 }
 
 void
@@ -351,17 +334,17 @@ CServerApp::closeServer(CServer* server)
 
 	// wait for clients to disconnect for up to timeout seconds
 	double timeout = 3.0;
-	CEventQueueTimer* timer = EVENTQUEUE->newOneShotTimer(timeout, NULL);
-	EVENTQUEUE->adoptHandler(CEvent::kTimer, timer,
+	CEventQueueTimer* timer = m_events->newOneShotTimer(timeout, NULL);
+	m_events->adoptHandler(CEvent::kTimer, timer,
 		new TMethodEventJob<CServerApp>(this, &CServerApp::handleClientsDisconnected));
-	EVENTQUEUE->adoptHandler(CServer::getDisconnectedEvent(), server,
+	m_events->adoptHandler(m_events->forCServer().disconnected(), server,
 		new TMethodEventJob<CServerApp>(this, &CServerApp::handleClientsDisconnected));
 	
-	EVENTQUEUE->loop();
+	m_events->loop();
 
-	EVENTQUEUE->removeHandler(CEvent::kTimer, timer);
-	EVENTQUEUE->deleteTimer(timer);
-	EVENTQUEUE->removeHandler(CServer::getDisconnectedEvent(), server);
+	m_events->removeHandler(CEvent::kTimer, timer);
+	m_events->deleteTimer(timer);
+	m_events->removeHandler(m_events->forCServer().disconnected(), server);
 
 	// done with server
 	delete server;
@@ -371,8 +354,8 @@ void
 CServerApp::stopRetryTimer()
 {
 	if (s_timer != NULL) {
-		EVENTQUEUE->deleteTimer(s_timer);
-		EVENTQUEUE->removeHandler(CEvent::kTimer, NULL);
+		m_events->deleteTimer(s_timer);
+		m_events->removeHandler(CEvent::kTimer, NULL);
 		s_timer = NULL;
 	}
 }
@@ -395,7 +378,7 @@ void
 CServerApp::closeClientListener(CClientListener* listen)
 {
 	if (listen != NULL) {
-		EVENTQUEUE->removeHandler(CClientListener::getConnectedEvent(), listen);
+		m_events->removeHandler(m_events->forCClientListener().connected(), listen);
 		delete listen;
 	}
 }
@@ -428,11 +411,11 @@ void
 CServerApp::closeServerScreen(CScreen* screen)
 {
 	if (screen != NULL) {
-		EVENTQUEUE->removeHandler(IScreen::getErrorEvent(),
+		m_events->removeHandler(m_events->forIScreen().error(),
 			screen->getEventTarget());
-		EVENTQUEUE->removeHandler(IScreen::getSuspendEvent(),
+		m_events->removeHandler(m_events->forIScreen().suspend(),
 			screen->getEventTarget());
-		EVENTQUEUE->removeHandler(IScreen::getResumeEvent(),
+		m_events->removeHandler(m_events->forIScreen().resume(),
 			screen->getEventTarget());
 		delete screen;
 	}
@@ -477,7 +460,7 @@ CServerApp::retryHandler(const CEvent&, void*)
 		LOG((CLOG_DEBUG1 "retry server initialization"));
 		s_serverState = kUninitialized;
 		if (!initServer()) {
-			EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+			m_events->addEvent(CEvent(CEvent::kQuit));
 		}
 		break;
 
@@ -485,12 +468,12 @@ CServerApp::retryHandler(const CEvent&, void*)
 		LOG((CLOG_DEBUG1 "retry server initialization"));
 		s_serverState = kUninitialized;
 		if (!initServer()) {
-			EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+			m_events->addEvent(CEvent(CEvent::kQuit));
 		}
 		else if (s_serverState == kInitialized) {
 			LOG((CLOG_DEBUG1 "starting server"));
 			if (!startServer()) {
-				EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+				m_events->addEvent(CEvent(CEvent::kQuit));
 			}
 		}
 		break;
@@ -499,7 +482,7 @@ CServerApp::retryHandler(const CEvent&, void*)
 		LOG((CLOG_DEBUG1 "retry starting server"));
 		s_serverState = kInitialized;
 		if (!startServer()) {
-			EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+			m_events->addEvent(CEvent(CEvent::kQuit));
 		}
 		break;
 	}
@@ -549,8 +532,8 @@ bool CServerApp::initServer()
 		// install a timer and handler to retry later
 		assert(s_timer == NULL);
 		LOG((CLOG_DEBUG "retry in %.0f seconds", retryTime));
-		s_timer = EVENTQUEUE->newOneShotTimer(retryTime, NULL);
-		EVENTQUEUE->adoptHandler(CEvent::kTimer, s_timer,
+		s_timer = m_events->newOneShotTimer(retryTime, NULL);
+		m_events->adoptHandler(CEvent::kTimer, s_timer,
 			new TMethodEventJob<CServerApp>(this, &CServerApp::retryHandler));
 		s_serverState = kInitializing;
 		return true;
@@ -564,15 +547,16 @@ bool CServerApp::initServer()
 CScreen* CServerApp::openServerScreen()
 {
 	CScreen* screen = createScreen();
-	EVENTQUEUE->adoptHandler(IScreen::getErrorEvent(),
+	screen->setEnableDragDrop(argsBase().m_enableDragDrop);
+	m_events->adoptHandler(m_events->forIScreen().error(),
 		screen->getEventTarget(),
 		new TMethodEventJob<CServerApp>(
 		this, &CServerApp::handleScreenError));
-	EVENTQUEUE->adoptHandler(IScreen::getSuspendEvent(),
+	m_events->adoptHandler(m_events->forIScreen().suspend(),
 		screen->getEventTarget(),
 		new TMethodEventJob<CServerApp>(
 		this, &CServerApp::handleSuspend));
-	EVENTQUEUE->adoptHandler(IScreen::getResumeEvent(),
+	m_events->adoptHandler(m_events->forIScreen().resume(),
 		screen->getEventTarget(),
 		new TMethodEventJob<CServerApp>(
 		this, &CServerApp::handleResume));
@@ -629,8 +613,8 @@ CServerApp::startServer()
 		// install a timer and handler to retry later
 		assert(s_timer == NULL);
 		LOG((CLOG_DEBUG "retry in %.0f seconds", retryTime));
-		s_timer = EVENTQUEUE->newOneShotTimer(retryTime, NULL);
-		EVENTQUEUE->adoptHandler(CEvent::kTimer, s_timer,
+		s_timer = m_events->newOneShotTimer(retryTime, NULL);
+		m_events->adoptHandler(CEvent::kTimer, s_timer,
 			new TMethodEventJob<CServerApp>(this, &CServerApp::retryHandler));
 		s_serverState = kStarting;
 		return true;
@@ -646,12 +630,12 @@ CServerApp::createScreen()
 {
 #if WINAPI_MSWINDOWS
 	return new CScreen(new CMSWindowsScreen(
-		true, args().m_noHooks, args().m_gameDevice, args().m_stopOnDeskSwitch));
+		true, args().m_noHooks, args().m_stopOnDeskSwitch, m_events), m_events);
 #elif WINAPI_XWINDOWS
 	return new CScreen(new CXWindowsScreen(
-		args().m_display, true, args().m_disableXInitThreads, 0, *EVENTQUEUE));
+		args().m_display, true, args().m_disableXInitThreads, 0, m_events), m_events);
 #elif WINAPI_CARBON
-	return new CScreen(new COSXScreen(true));
+	return new CScreen(new COSXScreen(m_events, true), m_events);
 #endif
 }
 
@@ -667,7 +651,7 @@ void
 CServerApp::handleScreenError(const CEvent&, void*)
 {
 	LOG((CLOG_CRIT "error on screen"));
-	EVENTQUEUE->addEvent(CEvent(CEvent::kQuit));
+	m_events->addEvent(CEvent(CEvent::kQuit));
 }
 
 void 
@@ -693,26 +677,32 @@ CServerApp::handleResume(const CEvent&, void*)
 CClientListener*
 CServerApp::openClientListener(const CNetworkAddress& address)
 {
-	CClientListener* listen =
-		new CClientListener(address, new CTCPSocketFactory, NULL);
-	EVENTQUEUE->adoptHandler(CClientListener::getConnectedEvent(), listen,
+	CClientListener* listen = new CClientListener(
+		address,
+		new CTCPSocketFactory(m_events, getSocketMultiplexer()),
+		NULL,
+		args().m_crypto,
+		m_events);
+	
+	m_events->adoptHandler(
+		m_events->forCClientListener().connected(), listen,
 		new TMethodEventJob<CServerApp>(
-		this, &CServerApp::handleClientConnected, listen));
+			this, &CServerApp::handleClientConnected, listen));
+	
 	return listen;
 }
 
 CServer* 
-CServerApp::openServer(const CConfig& config, CPrimaryClient* primaryClient)
+CServerApp::openServer(CConfig& config, CPrimaryClient* primaryClient)
 {
-	CServer* server = new CServer(config, primaryClient, s_serverScreen);
-
+	CServer* server = new CServer(config, primaryClient, s_serverScreen, m_events);
 	try {
-		EVENTQUEUE->adoptHandler(
-			CServer::getDisconnectedEvent(), server,
+		m_events->adoptHandler(
+			m_events->forCServer().disconnected(), server,
 			new TMethodEventJob<CServerApp>(this, &CServerApp::handleNoClients));
 
-		EVENTQUEUE->adoptHandler(
-			CServer::getScreenSwitchedEvent(), server,
+		m_events->adoptHandler(
+			m_events->forCServer().screenSwitched(), server,
 			new TMethodEventJob<CServerApp>(this, &CServerApp::handleScreenSwitched));
 
 	} catch (std::bad_alloc &ba) {
@@ -732,24 +722,6 @@ CServerApp::handleNoClients(const CEvent&, void*)
 void
 CServerApp::handleScreenSwitched(const CEvent& e, void*)
 {
-	if (!args().m_enableVnc)
-		return;
-
-	if (m_vncClient != NULL) {
-		LOG((CLOG_DEBUG "hiding vnc viewer for: %s", m_vncClient->m_screen.c_str()));
-		m_vncClient->hideViewer();
-	}
-
-	CServer::CSwitchToScreenInfo* info = reinterpret_cast<CServer::CSwitchToScreenInfo*>(e.getData());
-	std::map<CString, CVncClient*>::iterator it = m_vncClients.find(info->m_screen);
-	if (it == m_vncClients.end()) {
-		LOG((CLOG_DEBUG "could not find vnc client for: %s", info->m_screen));
-		return;
-	}
-
-	LOG((CLOG_DEBUG "showing vnc viewer for: %s", info->m_screen));
-	m_vncClient = it->second;
-	m_vncClient->showViewer();
 }
 
 int
@@ -758,6 +730,7 @@ CServerApp::mainLoop()
 	// create socket multiplexer.  this must happen after daemonization
 	// on unix because threads evaporate across a fork().
 	CSocketMultiplexer multiplexer;
+	setSocketMultiplexer(&multiplexer);
 
 	// if configuration has no screens then add this system
 	// as the default
@@ -792,39 +765,54 @@ CServerApp::mainLoop()
 	}
 
 	// load all available plugins.
-	ARCH->plugin().init(s_serverScreen->getEventTarget());
+	ARCH->plugin().init(s_serverScreen->getEventTarget(), m_events);
 
 	// handle hangup signal by reloading the server's configuration
 	ARCH->setSignalHandler(CArch::kHANGUP, &reloadSignalHandler, NULL);
-	EVENTQUEUE->adoptHandler(getReloadConfigEvent(),
-		IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(m_events->forCServerApp().reloadConfig(),
+		m_events->getSystemTarget(),
 		new TMethodEventJob<CServerApp>(this, &CServerApp::reloadConfig));
 
 	// handle force reconnect event by disconnecting clients.  they'll
 	// reconnect automatically.
-	EVENTQUEUE->adoptHandler(getForceReconnectEvent(),
-		IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(m_events->forCServerApp().forceReconnect(),
+		m_events->getSystemTarget(),
 		new TMethodEventJob<CServerApp>(this, &CServerApp::forceReconnect));
 
 	// to work around the sticky meta keys problem, we'll give users
 	// the option to reset the state of synergys
-	EVENTQUEUE->adoptHandler(getResetServerEvent(),
-		IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(m_events->forCServerApp().resetServer(),
+		m_events->getSystemTarget(),
 		new TMethodEventJob<CServerApp>(this, &CServerApp::resetServer));
 
 	// run event loop.  if startServer() failed we're supposed to retry
 	// later.  the timer installed by startServer() will take care of
 	// that.
 	DAEMON_RUNNING(true);
-	EVENTQUEUE->loop();
+	
+#if defined(MAC_OS_X_VERSION_10_7)
+	
+	CThread thread(
+		new TMethodJob<CServerApp>(
+			this, &CServerApp::runEventsLoop,
+			NULL));
+	
+	// HACK: sleep, allow queue to start.
+	ARCH->sleep(1);
+	
+	runCocoaApp();
+#else
+	m_events->loop();
+#endif
+	
 	DAEMON_RUNNING(false);
 
 	// close down
 	LOG((CLOG_DEBUG1 "stopping server"));
-	EVENTQUEUE->removeHandler(getForceReconnectEvent(),
-		IEventQueue::getSystemTarget());
-	EVENTQUEUE->removeHandler(getReloadConfigEvent(),
-		IEventQueue::getSystemTarget());
+	m_events->removeHandler(m_events->forCServerApp().forceReconnect(),
+		m_events->getSystemTarget());
+	m_events->removeHandler(m_events->forCServerApp().reloadConfig(),
+		m_events->getSystemTarget());
 	cleanupServer();
 	updateStatus();
 	LOG((CLOG_NOTE "stopped server"));
@@ -849,7 +837,7 @@ CServerApp::runInner(int argc, char** argv, ILogOutputter* outputter, StartupFun
 {
 	// general initialization
 	args().m_synergyAddress = new CNetworkAddress;
-	args().m_config         = new CConfig;
+	args().m_config         = new CConfig(m_events);
 	args().m_pname          = ARCH->getBasename(argv[0]);
 
 	// install caller's output filter
