@@ -41,8 +41,7 @@ public:
 	//! @name manipulators
 	//@{
 
-	IPlatformScreen() { }
-	IPlatformScreen(IEventQueue& eventQueue) : IKeyState(eventQueue) { }
+	IPlatformScreen(IEventQueue* events) : IKeyState(events) { }
 
 	//! Enable screen
 	/*!
@@ -131,6 +130,9 @@ public:
 	*/
 	virtual void		setSequenceNumber(UInt32) = 0;
 
+	//! Change dragging status
+	virtual void		setDraggingStarted(bool started) = 0;
+
 	//@}
 	//! @name accessors
 	//@{
@@ -158,20 +160,14 @@ public:
 	virtual void		fakeInputBegin() = 0;
 	virtual void		fakeInputEnd() = 0;
 	virtual SInt32		getJumpZoneSize() const = 0;
-	virtual bool		isAnyMouseButtonDown() const = 0;
+	virtual bool		isAnyMouseButtonDown(UInt32& buttonID) const = 0;
 	virtual void		getCursorCenter(SInt32& x, SInt32& y) const = 0;
-	virtual void		gameDeviceTimingResp(UInt16 freq) = 0;
-	virtual void		gameDeviceFeedback(GameDeviceID id, UInt16 m1, UInt16 m2) = 0;
 
 	// ISecondaryScreen overrides
 	virtual void		fakeMouseButton(ButtonID id, bool press) = 0;
-	virtual void		fakeMouseMove(SInt32 x, SInt32 y) const = 0;
+	virtual void		fakeMouseMove(SInt32 x, SInt32 y) = 0;
 	virtual void		fakeMouseRelativeMove(SInt32 dx, SInt32 dy) const = 0;
 	virtual void		fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const = 0;
-	virtual void		fakeGameDeviceButtons(GameDeviceID id, GameDeviceButton buttons) const = 0;
-	virtual void		fakeGameDeviceSticks(GameDeviceID id, SInt16 x1, SInt16 y1, SInt16 x2, SInt16 y2) const = 0;
-	virtual void		fakeGameDeviceTriggers(GameDeviceID id, UInt8 t1, UInt8 t2) const = 0;
-	virtual void		queueGameDeviceTimingReq() const = 0;
 
 	// IKeyState overrides
 	virtual void		updateKeyMap() = 0;
@@ -192,14 +188,22 @@ public:
 	virtual SInt32		pollActiveGroup() const = 0;
 	virtual void		pollPressedKeys(KeyButtonSet& pressedKeys) const = 0;
 
+	virtual CString&	getDraggingFileDir() = 0;
+	virtual bool		getDraggingStarted() = 0;
+	virtual bool		getFakeDraggingStarted() = 0;
+
+	virtual void		fakeDraggingFiles(CString str) = 0;
+	virtual const CString&
+						getDropTarget() const = 0;
+					
 protected:
 	//! Handle system event
 	/*!
 	A platform screen is expected to install a handler for system
 	events in its c'tor like so:
 	\code
-	EVENTQUEUE->adoptHandler(CEvent::kSystem,
-	 					 IEventQueue::getSystemTarget(),
+	m_events->adoptHandler(CEvent::kSystem,
+	 					 m_events->getSystemTarget(),
 	 					 new TMethodEventJob<CXXXPlatformScreen>(this,
 	 						 &CXXXPlatformScreen::handleSystemEvent));
 	\endcode
